@@ -6,7 +6,7 @@ use term::Term::*;
 use term::Error::*;
 use booleans::*;
 use pair::{pair, second};
-use combinators::z;
+use combinators::{i, k, z};
 
 /// Produces a Church-encoded number zero.
 ///
@@ -168,10 +168,10 @@ pub fn pow() -> Term {
 pub fn pred() -> Term {
     abs(abs(abs(
         app!(
-        Var(3),
-        abs(abs(app(Var(1), app(Var(2), Var(4))))),
-        abs(Var(2)),
-        abs(Var(1))
+            Var(3),
+            abs(abs(app(Var(1), app(Var(2), Var(4))))),
+            abs(Var(2)),
+            abs(Var(1))
         )
     )))
 }
@@ -366,8 +366,8 @@ pub fn gt() -> Term {
 /// Applied to two Church-encoded numbers it returns a Church-encoded pair with the result of their
 /// division - the quotient and the remainder. It loops indefinitely if the divisor is `zero()`.
 ///
-/// DIV := Y (λgqab.LT a b (PAIR q a) (g (SUCC q) (SUB a b) b)) ZERO =
-/// Y (λ λ λ λ LT 2 1 (PAIR 3 2) (4 (SUCC 3) (SUB 2 1) 1)) ZERO
+/// DIV := Z (λgqab.LT a b (λx.PAIR q a) (λx.g (SUCC q) (SUB a b) b) I) ZERO =
+/// Z (λ λ λ λ LT 2 1 (λ PAIR 4 3) (λ 5 (SUCC 4) (SUB 3 2) 2) I) ZERO
 ///
 /// # Example
 /// ```
@@ -391,13 +391,14 @@ pub fn div() -> Term {
                 lt(),
                 Var(2),
                 Var(1),
-                app!(pair(), Var(3), Var(2)),
-                app!(
-                    Var(4),
-                    app(succ(), Var(3)),
-                    app!(sub(), Var(2), Var(1)),
-                    Var(1)
-                )
+                abs(app!(pair(), Var(4), Var(3))),
+                abs(app!(
+                    Var(5),
+                    app(succ(), Var(4)),
+                    app!(sub(), Var(3), Var(2)),
+                    Var(2)
+                )),
+                i()
             )
         )))),
         zero()
@@ -407,8 +408,8 @@ pub fn div() -> Term {
 /// Applied to two Church-encoded numbers it returns a Church-encoded quotient of their division.
 /// It loops indefinitely if the second argument is `zero()`.
 ///
-/// QUOT := Y (λrab.LT a b ZERO (SUCC (r (SUB a b) b))) =
-/// Y (λ λ λ LT 2 1 ZERO (SUCC (3 (SUB 2 1) 1)))
+/// QUOT := Z (λrab.LT a b (λx.ZERO) (λx.SUCC (r (SUB a b) b)) I) =
+/// Z (λ λ λ LT 2 1 (λ ZERO) (λ SUCC (4 (SUB 3 2) 2)) I)
 ///
 /// # Example
 /// ```
@@ -431,15 +432,16 @@ pub fn quot() -> Term {
                 lt(),
                 Var(2),
                 Var(1),
-                zero(),
-                app(
+                abs(zero()),
+                abs(app(
                     succ(),
                     app!(
-                        Var(3),
-                        app!(sub(), Var(2), Var(1)),
-                        Var(1)
+                        Var(4),
+                        app!(sub(), Var(3), Var(2)),
+                        Var(2)
                     )
-                )
+                )),
+                i()
             )
         )))
     )
@@ -472,8 +474,8 @@ pub fn rem() -> Term {
 
 /// Applied to a Church-encoded number it yields its Church-encoded factorial.
 ///
-/// FACTORIAL := Y (λgx. ISZERO x ONE (MULT x (g (PRED x)))) =
-/// Y (λ λ ISZERO 1 ONE (MULT 1 (2 (PRED 1))))
+/// FACTORIAL := λn. n (λfax. f (MULT a x) (SUCC x)) K ONE ONE =
+/// λ 1 (λ λ λ 3 (MULT 2 1) (SUCC 1)) K ONE ONE
 ///
 /// # Example
 /// ```
@@ -486,20 +488,20 @@ pub fn rem() -> Term {
 /// assert_eq!(expr, 6.into());
 /// ```
 pub fn factorial() -> Term {
-    app(
-        z(),
-        abs(abs(
-            app!(
-                is_zero(),
-                Var(1),
-                one(),
+    abs(
+        app!(
+            Var(1),
+            abs(abs(abs(
                 app!(
-                    mult(),
-                    Var(1),
-                    app(Var(2), app(pred(), Var(1)))
+                    Var(3),
+                    app!(mult(), Var(2), Var(1)),
+                    app!(succ(), Var(1))
                 )
-            )
-        ))
+            ))),
+            k(),
+            one(),
+            one()
+        )
     )
 }
 
@@ -698,12 +700,11 @@ mod test {
         assert_eq!(beta(app!(div(), 3.into(), 2.into()), &HybridNormal, 0), (1.into(), 1.into()).into());
         assert_eq!(beta(app!(div(), 2.into(), 1.into()), &HybridNormal, 0), (2.into(), 0.into()).into());
         assert_eq!(beta(app!(div(), 0.into(), 3.into()), &HybridNormal, 0), (0.into(), 0.into()).into());
-/*
+
         assert_eq!(beta(app!(div(), 2.into(), 2.into()), &HybridApplicative, 0), (1.into(), 0.into()).into());
         assert_eq!(beta(app!(div(), 3.into(), 2.into()), &HybridApplicative, 0), (1.into(), 1.into()).into());
         assert_eq!(beta(app!(div(), 2.into(), 1.into()), &HybridApplicative, 0), (2.into(), 0.into()).into());
         assert_eq!(beta(app!(div(), 0.into(), 3.into()), &HybridApplicative, 0), (0.into(), 0.into()).into());
-*/
     }
 
     #[test]
@@ -718,12 +719,11 @@ mod test {
         assert_eq!(beta(app!(quot(), 3.into(), 2.into()), &HybridNormal, 0), 1.into());
         assert_eq!(beta(app!(quot(), 2.into(), 1.into()), &HybridNormal, 0), 2.into());
         assert_eq!(beta(app!(quot(), 0.into(), 3.into()), &HybridNormal, 0), 0.into());
-/*
+
         assert_eq!(beta(app!(quot(), 2.into(), 2.into()), &HybridApplicative, 0), 1.into());
         assert_eq!(beta(app!(quot(), 3.into(), 2.into()), &HybridApplicative, 0), 1.into());
         assert_eq!(beta(app!(quot(), 2.into(), 1.into()), &HybridApplicative, 0), 2.into());
         assert_eq!(beta(app!(quot(), 0.into(), 3.into()), &HybridApplicative, 0), 0.into());
-*/
     }
 
     #[test]
@@ -738,12 +738,11 @@ mod test {
         assert_eq!(beta(app!(rem(), 3.into(), 2.into()), &HybridNormal, 0), 1.into());
         assert_eq!(beta(app!(rem(), 2.into(), 1.into()), &HybridNormal, 0), 0.into());
         assert_eq!(beta(app!(rem(), 0.into(), 3.into()), &HybridNormal, 0), 0.into());
-/*
+
         assert_eq!(beta(app!(rem(), 2.into(), 2.into()), &HybridApplicative, 0), 0.into());
         assert_eq!(beta(app!(rem(), 3.into(), 2.into()), &HybridApplicative, 0), 1.into());
         assert_eq!(beta(app!(rem(), 2.into(), 1.into()), &HybridApplicative, 0), 0.into());
         assert_eq!(beta(app!(rem(), 0.into(), 3.into()), &HybridApplicative, 0), 0.into());
-*/
     }
 
     #[test]
@@ -757,11 +756,10 @@ mod test {
         assert_eq!(beta(app!(factorial(), 1.into()), &HybridNormal, 0), 1.into());
         assert_eq!(beta(app!(factorial(), 2.into()), &HybridNormal, 0), 2.into());
         assert_eq!(beta(app!(factorial(), 3.into()), &HybridNormal, 0), 6.into());
-/*
+
         assert_eq!(beta(app!(factorial(), 0.into()), &HybridApplicative, 0), 1.into());
         assert_eq!(beta(app!(factorial(), 1.into()), &HybridApplicative, 0), 1.into());
         assert_eq!(beta(app!(factorial(), 2.into()), &HybridApplicative, 0), 2.into());
         assert_eq!(beta(app!(factorial(), 3.into()), &HybridApplicative, 0), 6.into());
-*/
     }
 }
