@@ -6,7 +6,7 @@ use term::Error::*;
 use booleans::*;
 use pair::*;
 use arithmetic::{zero, succ};
-use combinators::{y, i};
+use combinators::{i, z};
 use std::ops::Index;
 
 /// Equivalent to `booleans::fls()`; produces a Church-encoded `nil`, the last link of a
@@ -124,8 +124,8 @@ pub fn tail() -> Term { second() }
 
 /// Applied to a Church list it returns its Church-encoded length.
 ///
-/// LENGTH := Y (λgcx.NULL x c (g (SUCC c) (SECOND x))) ZERO
-/// = Y (λλλ NULL 1 2 (3 (SUCC 2) (SECOND 1))) ZERO
+/// LENGTH := Z (λgcx.NULL x (λz.c) (λz.g (SUCC c) (SECOND x)) I) ZERO
+/// = Z (λλλ NULL 1 (λ 3) (λ 4 (SUCC 3) (SECOND 2)) I) ZERO
 ///
 /// # Example
 /// ```
@@ -144,17 +144,18 @@ pub fn tail() -> Term { second() }
 /// ```
 pub fn length() -> Term {
     app!(
-        y(),
+        z(),
         abs(abs(abs(
             app!(
                 null(),
                 Var(1),
-                Var(2),
-                app!(
-                    Var(3),
-                    app(succ(), Var(2)),
-                    app(second(), Var(1))
-                )
+                abs(Var(3)),
+                abs(app!(
+                    Var(4),
+                    app(succ(), Var(3)),
+                    app(second(), Var(2))
+                )),
+                i()
             )
         ))),
         zero()
@@ -163,8 +164,8 @@ pub fn length() -> Term {
 
 /// Reverses a Church list.
 ///
-/// REVERSE := Y (λgal.NULL l a (g (PAIR (FIRST l) a) (SECOND l))) NIL =
-/// Y (λ λ λ NULL 1 2 (3 (PAIR (FIRST 1) 2) (SECOND 1))) NIL
+/// REVERSE := Z (λgal.NULL l (λx.a) (λx.g (PAIR (FIRST l) a) (SECOND l) I)) NIL =
+/// Z (λ λ λ NULL 1 (λ 3) (λ 4 (PAIR (FIRST 2) 3) (SECOND 2)) I) NIL
 ///
 /// # Example
 /// ```
@@ -178,22 +179,25 @@ pub fn length() -> Term {
 ///
 /// let list = Term::from(vec![one(), one(), zero()]);
 ///
-/// assert_eq!(beta(app!(reverse(), list), &Normal, 0), Term::from(vec![zero(), one(), one()]));
+/// assert_eq!(beta(app!(reverse(), list), &Normal, 0),
+///            Term::from(vec![zero(), one(), one()])
+/// );
 /// # }
 /// ```
 pub fn reverse() -> Term {
     app!(
-        y(),
+        z(),
         abs(abs(abs(
             app!(
                 null(),
                 Var(1),
-                Var(2),
-                app!(
-                    Var(3),
-                    app!(pair(), app(first(), Var(1)), Var(2)),
-                    app(second(), Var(1))
-                )
+                abs(Var(3)),
+                abs(app!(
+                    Var(4),
+                    app!(pair(), app(first(), Var(2)), Var(3)),
+                    app(second(), Var(2))
+                )),
+                i()
             )
         ))),
         nil()
@@ -233,8 +237,8 @@ pub fn list() -> Term {
 
 /// Applied to 2 Church lists it concatenates them.
 ///
-/// APPEND := Y (λgab. NULL a b (PAIR (FIRST a) (g (SECOND a) b))) =
-/// Y (λ λ λ NULL 2 1 (PAIR (FIRST 2) (3 (SECOND 2) 1)))
+/// APPEND := Z (λgab. NULL a (λx.b) (λx.PAIR (FIRST a) (g (SECOND a) b)) I) =
+/// Z (λ λ λ NULL 2 (λ 2) (λ PAIR (FIRST 3) (4 (SECOND 3) 2)) I)
 ///
 /// # Example
 /// ```
@@ -253,17 +257,18 @@ pub fn list() -> Term {
 /// # }
 /// ```
 pub fn append() -> Term {
-    y().app(
+    z().app(
         abs(abs(abs(
             app!(
                 null(),
                 Var(2),
-                Var(1),
-                app!(
+                abs(Var(2)),
+                abs(app!(
                     pair(),
-                    app(first(), Var(2)),
-                    app!(Var(3), app(second(), Var(2)), Var(1))
-                )
+                    app(first(), Var(3)),
+                    app!(Var(4), app(second(), Var(3)), Var(2))
+                )),
+                i()
             )
         )))
     )
@@ -297,8 +302,8 @@ pub fn index() -> Term {
 
 /// Applied to a function and a Church list it maps the function over it.
 ///
-/// MAP := Y (λgfx. NULL x NIL (PAIR (f (FIRST x)) (g f (SECOND x)))) =
-/// Y (λ λ λ NULL 1 NIL (PAIR (2 (FIRST 1)) (3 2 (SECOND 1))))
+/// MAP := Z (λgfx. NULL x (λa.NIL) (λa.PAIR (f (FIRST x)) (g f (SECOND x))) I) =
+/// Z (λ λ λ NULL 1 (λ NIL) (λ PAIR (3 (FIRST 2)) (4 3 (SECOND 2))) I)
 ///
 /// # Example
 /// ```
@@ -317,17 +322,18 @@ pub fn index() -> Term {
 /// # }
 /// ```
 pub fn map() -> Term {
-    y().app(
+    z().app(
         abs(abs(abs(
             app!(
                 null(),
                 Var(1),
-                nil(),
-                app!(
+                abs(nil()),
+                abs(app!(
                     pair(),
-                    app(Var(2), app(first(), Var(1))),
-                    app!(Var(3), Var(2), app(second(), Var(1)))
-                )
+                    app(Var(3), app(first(), Var(2))),
+                    app!(Var(4), Var(3), app(second(), Var(2)))
+                )),
+                i()
             )
         )))
     )
@@ -337,8 +343,8 @@ pub fn map() -> Term {
 /// [left fold](https://en.wikipedia.org/wiki/Fold_(higher-order_function)#Folds_on_lists) on the
 /// list.
 ///
-/// FOLDL := Y (λgfex. NULL x e (g f (f e (FIRST x)) (SECOND x))) =
-/// Y (λ λ λ λ NULL 1 2 (4 3 (3 2 (FIRST 1)) (SECOND 1)))
+/// FOLDL := Z (λgfex. NULL x (λz.e) (λz.g f (f e (FIRST x)) (SECOND x)) I) =
+/// Z (λ λ λ λ NULL 1 (λ 2) (λ 5 4 (4 3 (FIRST 2)) (SECOND 2)) I)
 ///
 /// # Example
 /// ```
@@ -357,18 +363,19 @@ pub fn map() -> Term {
 /// # }
 /// ```
 pub fn foldl() -> Term {
-    y().app(
+    z().app(
         abs(abs(abs(abs(
             app!(
                 null(),
                 Var(1),
-                Var(2),
-                app!(
+                abs(Var(3)),
+                abs(app!(
+                    Var(5),
                     Var(4),
-                    Var(3),
-                    app!(Var(3), Var(2), app(first(), Var(1))),
-                    app(second(), Var(1))
-                )
+                    app!(Var(4), Var(3), app(first(), Var(2))),
+                    app(second(), Var(2))
+                )),
+                i()
             )
         ))))
     )
@@ -378,8 +385,8 @@ pub fn foldl() -> Term {
 /// [right fold](https://en.wikipedia.org/wiki/Fold_(higher-order_function)#Folds_on_lists) on the
 /// list.
 ///
-/// FOLDR := λfex. Y (λgy. NULL y e (f (FIRST y) (g (SECOND y)))) x =
-/// λ λ λ Y (λ λ NULL 1 4 (5 (FIRST 1) (2 (SECOND 1)))) 1
+/// FOLDR := λfex. Z (λgy. NULL y (λz.e) (λz.f (FIRST y) (g (SECOND y))) I) x =
+/// λ λ λ Z (λ λ NULL 1 (λ 5) (λ 6 (FIRST 2) (3 (SECOND 2))) I) 1
 ///
 /// # Example
 /// ```
@@ -400,17 +407,18 @@ pub fn foldl() -> Term {
 pub fn foldr() -> Term {
     abs(abs(abs(
         app!(
-            y(),
+            z(),
             abs(abs(
                 app!(
                     null(),
                     Var(1),
-                    Var(4),
-                    app!(
-                        Var(5),
-                        app(first(), Var(1)),
-                        app(Var(2), app(second(), Var(1)))
-                    )
+                    abs(Var(5)),
+                    abs(app!(
+                        Var(6),
+                        app(first(), Var(2)),
+                        app(Var(3), app(second(), Var(2)))
+                    )),
+                    i()
                 )
             )),
             Var(1)
@@ -420,8 +428,8 @@ pub fn foldr() -> Term {
 
 /// Applied to a predicate and a Church list it filters the list based on the predicate.
 ///
-/// FILTER := Y (λgfx. NULL x NIL (f (FIRST x) (PAIR (FIRST x)) I (g f (SECOND x)))) =
-/// Y (λ λ λ NULL 1 NIL (2 (FIRST 1) (PAIR (FIRST 1)) I (3 2 (SECOND 1))))
+/// FILTER := Z (λgfx. NULL x (λz.NIL) (λz.f (FIRST x) (PAIR (FIRST x)) I (g f (SECOND x))) I) =
+/// Z (λ λ λ NULL 1 (λ NIL) (λ 3 (FIRST 2) (PAIR (FIRST 2)) I (4 3 (SECOND 2))) I)
 ///
 /// # Example
 /// ```
@@ -444,19 +452,20 @@ pub fn foldr() -> Term {
 /// # }
 /// ```
 pub fn filter() -> Term {
-    y().app(
+    z().app(
         abs(abs(abs(
             app!(
                 null(),
                 Var(1),
-                nil(),
-                app!(
-                    Var(2),
-                    app(first(), Var(1)),
-                    app(pair(), app(first(), Var(1))),
+                abs(nil()),
+                abs(app!(
+                    Var(3),
+                    app(first(), Var(2)),
+                    app(pair(), app(first(), Var(2))),
                     i(),
-                    app!(Var(3), Var(2), app(second(), Var(1)))
-                )
+                    app!(Var(4), Var(3), app(second(), Var(2)))
+                )),
+                i()
             )
         )))
     )
