@@ -3,14 +3,23 @@
 use term::*;
 use term::Term::*;
 use self::Order::*;
+use std::fmt;
 
 /// Set to `true` to see all the steps of β-reductions. The default is `false`.
 pub const SHOW_REDUCTIONS: bool = false;
 
 /// The [evaluation order](http://www.cs.cornell.edu/courses/cs6110/2014sp/Handouts/Sestoft.pdf) of
-/// β-reductions. `Applicative` order will fail to fully reduce expressions containing functions
-/// without a normal form, e.g. the Y combinator (they will expand forever). `CallByName`,
-/// `HeadSpine` and `CallByValue` orders don't always normalize fully. The default is `Normal`.
+/// β-reductions. The default is `Normal`.
+///
+/// They don't always yield the same result:
+///
+/// - The `Normal`, `HybridNormal`, `Applicative` and `HybridApplicative` orders reduce expressions
+/// to their normal forms
+/// - The `Applicative` order will fail to fully reduce expressions containing functions
+/// without a normal form, e.g. the `Y` combinator (they will expand forever)
+/// - The `CallByName` order reduces to weak head normal form
+/// - The `CallByValue` order reduces to weak normal form
+/// - The `HeadSpine` order reduces to head normal form
 #[derive(Debug, PartialEq)]
 pub enum Order {
     /// leftmost outermost
@@ -19,13 +28,13 @@ pub enum Order {
     CallByName,
     /// leftmost outermost, but abstractions reduced only in head position
     HeadSpine,
-    /// `HeadSpine` + `Normal`
+    /// a hybrid between `HeadSpine` and `Normal`
     HybridNormal,
     /// leftmost innermost
     Applicative,
     /// leftmost innermost, but not inside abstractions
     CallByValue,
-    /// `CallByValue` + `Applicative`
+    /// a hybrid between `CallByValue` and `Applicative`
     HybridApplicative
 }
 
@@ -101,6 +110,64 @@ pub fn beta(mut term: Term, order: &Order, limit: usize) -> Term {
     term
 }
 
+/// Prints the number of reductions required for `term` to reach the final form with all the
+/// available reduction strategies, optionally excluding the ones from the given list. Such
+/// exclusions might be necessary, especially if the expression contains the fixed-point combinator.
+///
+/// # Example
+///
+/// ```
+/// use lambda_calculus::arithmetic::factorial;
+/// use lambda_calculus::reduction::benchmark;
+/// use lambda_calculus::reduction::Order::*;
+///
+/// benchmark(&factorial().app(3.into()), &[CallByName, CallByValue, HeadSpine]);
+///
+/// // stdout:
+///
+/// // normal:             46
+/// // applicative:        39
+/// // hybrid normal:      46
+/// // hybrid applicative: 39
+/// ```
+pub fn benchmark(term: &Term, exclude: &[Order]) {
+    let mut count = 0;
+    if !exclude.contains(&CallByName) {
+        term.clone().beta_cbn(0, 0, &mut count);
+        println!("{}:       {}", CallByName, count);
+        count = 0;
+    }
+    if !exclude.contains(&Normal) {
+        term.clone().beta_nor(0, 0, &mut count);
+        println!("{}:             {}", Normal, count);
+        count = 0;
+    }
+    if !exclude.contains(&CallByValue) {
+        term.clone().beta_cbv(0, 0, &mut count);
+        println!("{}:      {}", CallByValue, count);
+        count = 0;
+    }
+    if !exclude.contains(&Applicative) {
+        term.clone().beta_app(0, 0, &mut count);
+        println!("{}:        {}", Applicative, count);
+        count = 0;
+    }
+    if !exclude.contains(&HeadSpine) {
+        term.clone().beta_hs(0, 0, &mut count);
+        println!("{}:         {}", HeadSpine, count);
+        count = 0;
+    }
+    if !exclude.contains(&HybridNormal) {
+        term.clone().beta_hnor(0, 0, &mut count);
+        println!("{}:      {}", HybridNormal, count);
+        count = 0;
+    }
+    if !exclude.contains(&HybridApplicative) {
+        term.clone().beta_happ(0, 0, &mut count);
+        println!("{}: {}", HybridApplicative, count);
+    }
+}
+
 impl Term {
     /// Applies `self` to another `Term` and performs substitution, consuming `self` in the process.
     /// It produces an `Error` if `self` is not an `Abs`traction.
@@ -138,7 +205,12 @@ impl Term {
     }
 
     fn eval_with_info(&mut self, depth: u32, count: &usize) {
-        if SHOW_REDUCTIONS { print!("\n{}. {} reduces to ", count + 1, show_precedence(self, 0, depth)) };
+        if SHOW_REDUCTIONS {
+            print!("\n{}. {}\n=>", count + 1, show_precedence(self, 0, depth));
+            let mut indent_len = ((*count + 1) as f32).log10().trunc() as usize + 3;
+            if DISPLAY_CLASSIC { indent_len += 3 }
+            for _ in 0..indent_len { print!(" ") };
+        };
         let copy = self.clone();
         *self = copy.eval().unwrap();
         if SHOW_REDUCTIONS { println!("{}", show_precedence(self, 0, depth)) };
@@ -164,7 +236,17 @@ impl Term {
     /// assert_eq!(pred_one, 0.into());
     /// ```
     pub fn beta(&mut self, order: &Order, limit: usize) {
-        if SHOW_REDUCTIONS { println!("reducing {}:", self) };
+        if SHOW_REDUCTIONS {
+            println!("reducing {} [{} order{}]:", self, order,
+                if limit != 0 {
+                    format!(", limit of {} reduction{}", limit,
+                        if limit == 1 { "" } else { "s" }
+                    )
+                } else {
+                    "".into()
+                }
+            );
+        };
 
         let mut count = 0;
 
@@ -177,7 +259,10 @@ impl Term {
             HybridNormal      => self.beta_hnor(0, limit, &mut count),
             HybridApplicative => self.beta_happ(0, limit, &mut count)
         }
-        if SHOW_REDUCTIONS { println!("\nfinal form: {}", self) };
+        if SHOW_REDUCTIONS {
+            println!("\nresult after {} reduction{}: {}\n", count,
+                if limit == 1 { "" } else { "s" }, self);
+        };
     }
 
     fn beta_cbn(&mut self, depth: u32, limit: usize, count: &mut usize) {
@@ -316,6 +401,20 @@ impl Term {
     }
 }
 
+impl fmt::Display for Order {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(f, "{}", match *self {
+            Normal => "normal",
+            CallByName => "call-by-name",
+            HeadSpine => "head spine",
+            HybridNormal => "hybrid normal",
+            Applicative => "applicative",
+            CallByValue => "call-by-value",
+            HybridApplicative => "hybrid applicative"
+        })
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
